@@ -1,5 +1,7 @@
+import asyncio
 import json
 import logging
+import pickle
 
 import socketio
 import ujson as ujson
@@ -16,20 +18,24 @@ class GSCNodeFunctionReceiverSocket:
         self.oncnodefunction = oncnodefunction
         self._logger = logging.getLogger(__name__)
 
+        @self.sio.event
+        def connect():
+            self._logger.info('oncnodefunction connected')
+
         def on_success(r):
             self._logger.info('cnodefunction succeed')
 
         def on_errorPost(error):
             self._logger.exception('cnodefunction failed :' + error)
 
-        @self.sio.event
-        def connect():
-            self._logger.info('oncnodefunction connected')
-
         @self.sio.event
         def oncnodefunction(id, msg):
-            self._pool.apply_async(self.oncnodefunction, args=[self.gsdbs, id, ujson.loads(msg)], callback=on_success,
-                                   error_callback=on_errorPost)
+            msg1 = pickle.loads(msg)
+            if msg1["cnodefunction"] == self.gsdbs.credentials["cnode"]:
+                self._pool.apply_async(self.oncnodefunction, args=[self.gsdbs, id, pickle.loads(msg)],
+                                       callback=on_success,
+                                       error_callback=on_errorPost)
+            # self.oncnodefunction(self.gsdbs, id, ujson.loads(msg))
 
         connectURL = ""
 
@@ -93,9 +99,24 @@ class GSCNodeFunctionCaller:
         self.sio = socketio.Client()
         self.gsdbs = gsdbs
         self.oncnodefunctionanswer = oncnodefunctionanswer
+        self._pool = Pool(100)
+        self._logger = logging.getLogger(__name__)
 
     def startSocket(self):
         GSCNodeFunctionCallerSocket.create(self.sio, self.gsdbs, self.oncnodefunctionanswer)
 
+    def on_success(self, r):
+        self._logger.info('cnodefunction succeed')
+
+    def on_errorPost(self, error):
+        self._logger.exception('cnodefunction failed :' + error)
+
+    def emitFunction(self, sio, target, msg):
+        # sio.emit("oncnodefunction", {"target": target, "message": ujson.dumps(msg)})
+        msg["cnodefunction"] = target
+        data_string = pickle.dumps(msg)
+        sio.emit("oncnodefunction", data_string)
+
     def sendcnodefunction(self, target, msg):
-        self.sio.emit("oncnodefunction", {"target": target, "message": json.dumps(msg)})
+        self._pool.apply_async(self.emitFunction, args=[self.sio, target, msg], callback=self.on_success,
+                               error_callback=self.on_errorPost)
